/*
 * File: MyGame.js 
 * This is the logic of our game. For now, this is very simple.
 */
/*jslint node: true, vars: true */
/*global gEngine: false, SimpleShader: false, Renderable: false, Camera: false, mat4: false, vec3: false, vec2: false */
/* find out more about jslint: http://www.jslint.com/help.html */

"use strict";  // Operate in Strict mode such that variables must be declared before used!

function ClassExample(htmlCanvasID) {
    // variables of the shader for drawing: one shader to be shared by two renderables
    this.mConstColorShader = null;

    // variables for the squares
    this.mCurrentObject = null;        // these are the Renderable objects
    this.mAllObjects = [];             // Initialize the array

    gEngine.Core.initializeWebGL(htmlCanvasID);

    this.mConstColorShader = new SimpleShader(
        "src/GLSLShaders/SimpleVS.glsl",      // Path to the VertexShader 
        "src/GLSLShaders/SimpleFS.glsl");    // Path to the simple FragmentShader

    // create first object
    this.defineCenter(400, 300);
    this.mCurrentObject.getXform().setSize(20, 20);
    
    this.mAccel = [0, -0.5]; // downward acceleration
}

ClassExample.prototype.draw = function (camera) {

    // Step E: Clear the canvas
    gEngine.Core.clearCanvas([0.9, 0.9, 0.9, 1]);        // Clear the canvas

    // Step F: Starts the drawing by activating the camera
    camera.setupViewProjection();

    var i;
    for (i=0; i<this.mAllObjects.length; i++)
        this.mAllObjects[i].draw(camera);
};

ClassExample.prototype.update = function () {
    var i, j, xf, v;
    for (i=0; i<this.mAllObjects.length; i++) {
        xf = this.mAllObjects[i].update(this.mAccel);
    }
    
    // bounces square around in the window
    for (i=this.mAllObjects.length-1; i>=0; i--) {
        v = this.mAllObjects[i].getVelocity();
        xf = this.mAllObjects[i].getXform();
        if ( (xf.getYPos() < 0.0) || (xf.getYPos() > 600) ) {
            v[1] = -v[1] * 0.9;
        } 
        
        if ( (xf.getXPos() < 0) || (xf.getXPos() > 800)) {
            v[0] = -v[0];
        }
        
        // should stop some 
        if ((xf.getYPos() < 10) && Math.abs(v[1]) < 5) {
            v[1] = 0;
            xf.setYPos(0);
        }
    }
    
    var target;
    for (i=this.mAllObjects.length-1; i>=0; i--) {
        target = this.mAllObjects[i];
        for (j = this.mAllObjects.length-1; j>i; j--) {
            if (this.mAllObjects[j].mayHaveCollided(target)) {
                this.mAllObjects.splice(j, 1);
            }
        }
    }
};